package br.gov.mj.side.web.service;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.inject.Inject;

import org.apache.commons.lang3.StringUtils;

import br.gov.mj.infra.negocio.exception.BusinessException;
import br.gov.mj.infra.negocio.persistencia.EnumOrder;
import br.gov.mj.infra.negocio.persistencia.IGenericPersister;
import br.gov.mj.seg.entidades.Usuario;
import br.gov.mj.side.entidades.entidade.Entidade;
import br.gov.mj.side.entidades.entidade.LocalEntregaEntidade;
import br.gov.mj.side.entidades.entidade.Pessoa;
import br.gov.mj.side.entidades.entidade.PessoaEntidade;
import br.gov.mj.side.entidades.enums.EnumStatusLocalEntrega;
import br.gov.mj.side.web.dao.EntidadeDAO;
import br.gov.mj.side.web.dto.EntidadePesquisaDto;

@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
public class EntidadeService {

    @Inject
    private IGenericPersister genericPersister;

    @Inject
    private EntidadeDAO entidadeDAO;

    @Inject
    private UsuarioService usuarioService;

    @Inject
    private InscricaoProgramaService inscricaoProgramaService;

    public List<LocalEntregaEntidade> buscarLocaisEntrega(Entidade entidade, EnumStatusLocalEntrega statusLocalEntrega) {

        if (entidade == null || entidade.getId() == null) {
            throw new IllegalArgumentException("Parâmetro id não pode ser null");
        }

        List<LocalEntregaEntidade> lista = entidadeDAO.buscarLocaisEntrega(entidade.getId(), statusLocalEntrega);
        List<LocalEntregaEntidade> listaRetorno = new ArrayList<LocalEntregaEntidade>();
        for (LocalEntregaEntidade localEntregaEntidade : lista) {
            localEntregaEntidade.setPossuiVinculo(possuiVinculo(localEntregaEntidade));
            listaRetorno.add(localEntregaEntidade);
        }
        return listaRetorno;

    }

    public LocalEntregaEntidade buscarLocalEntregaEntidadePeloId(Long id) {
        LocalEntregaEntidade localEntregaEntidade = entidadeDAO.buscarLocalEntregaEntidadePeloId(id);
        localEntregaEntidade.setPossuiVinculo(possuiVinculo(localEntregaEntidade));
        return localEntregaEntidade;
    }

    public Entidade sincronizarLocaisDeEntrega(Entidade entidade, String usuarioLogado) {

        Entidade entidadeRetorno = null;

        validarObjetoNulo(entidade);

        BusinessException ex = new BusinessException();

        if (StringUtils.isBlank(usuarioLogado)) {
            ex.addErrorMessage("Usuario logado obrigatório");
            throw ex;
        }

        validarLocalEntregaComMesmoNome(entidade, ex);
        entidadeRetorno = entidadeDAO.sincronizarLocaisDeEntrega(entidade, usuarioLogado);
        return entidadeRetorno;
    }

    public List<PessoaEntidade> buscarEntidade(Pessoa pessoa) {

        if (pessoa == null || pessoa.getId() == null) {
            throw new IllegalArgumentException("Parâmetro id não pode ser null");
        }
        return entidadeDAO.buscarEntidade(pessoa.getId());
    }

    public List<PessoaEntidade> buscarPessoa(Entidade entidade) {

        if (entidade == null || entidade.getId() == null) {
            throw new IllegalArgumentException("Parâmetro id não pode ser null");
        }

        return entidadeDAO.buscarPessoa(entidade.getId());
    }

    public List<PessoaEntidade> buscarPessoaEntidadesDoUsuario(Usuario usuario) {

        if (usuario == null || usuario.getId() == null) {
            throw new IllegalArgumentException("Parâmetro id não pode ser null");
        }

        return entidadeDAO.buscarEntidadesDoUsuario(usuario.getId());
    }

    // Adicionado momentaneamente para buscar apenas uma entidade por
    // representante
    // No futuro um representante poderá ter mais de uma entidade.
    public Entidade buscarEntidadeDoUsuario(Usuario usuario) {
        List<PessoaEntidade> entidades = buscarPessoaEntidadesDoUsuario(usuario);
        if (!entidades.isEmpty()) {
            Pessoa pessoa = entidades.get(0).getPessoa();
            if (pessoa.isRepresentante()) {
                return entidades.get(0).getEntidade();
            }
        }
        return null;
    }

    // Adicionado momentaneamente para buscar apenas uma entidade por
    // representante
    // No futuro um representante poderá ter mais de uma entidade.
    public PessoaEntidade buscarPessoaEntidadeDoUsuario(Usuario usuario) {
        List<PessoaEntidade> entidades = buscarPessoaEntidadesDoUsuario(usuario);
        if (!entidades.isEmpty()) {
            if (entidades.get(0).getPessoa().isRepresentante()) {
                return entidades.get(0);
            }
        }
        return null;
    }

    public List<PessoaEntidade> buscarRepresentanteEntidade(Entidade entidade, boolean somenteAtivos) {

        if (entidade == null || entidade.getId() == null) {
            throw new IllegalArgumentException("Parâmetro id não pode ser null");
        }
        return entidadeDAO.buscarRepresentanteEntidade(entidade.getId(), somenteAtivos);
    }

    public List<PessoaEntidade> buscarTitularEntidade(Entidade entidade) {
        if (entidade == null || entidade.getId() == null) {
            throw new IllegalArgumentException("Parâmetro id não pode ser null");
        }

        return entidadeDAO.buscarTitularEntidade(entidade.getId());
    }

    public List<Entidade> buscarPaginado(EntidadePesquisaDto entidadePesquisaDto, int first, int size, EnumOrder order, String propertyOrder) {

        BusinessException ex = new BusinessException();

        if (entidadePesquisaDto.getUsuarioLogado() == null || entidadePesquisaDto.getUsuarioLogado().getId() == null) {
            ex.addErrorMessage("Usuario logado obrigatório");
            throw ex;
        }

        return entidadeDAO.buscarPaginado(entidadePesquisaDto, first, size, order, propertyOrder);
    }

    public List<Entidade> buscarSemPaginacao(EntidadePesquisaDto entidadePesquisaDto) {

        BusinessException ex = new BusinessException();

        if (entidadePesquisaDto.getUsuarioLogado() == null || entidadePesquisaDto.getUsuarioLogado().getId() == null) {
            ex.addErrorMessage("Usuario logado obrigatório");
            throw ex;
        }

        return entidadeDAO.buscarSemPaginacao(entidadePesquisaDto);
    }

    public List<Entidade> buscarSemPaginacaoOrdenado(EntidadePesquisaDto entidadePesquisaDto, EnumOrder order, String propertyOrder) {

        BusinessException ex = new BusinessException();

        if (entidadePesquisaDto.getUsuarioLogado() == null || entidadePesquisaDto.getUsuarioLogado().getId() == null) {
            ex.addErrorMessage("Usuario logado obrigatório");
            throw ex;
        }

        return entidadeDAO.buscarSemPaginacaoOrdenado(entidadePesquisaDto, order, propertyOrder);
    }

    public List<Entidade> buscar(EntidadePesquisaDto entidadePesquisaDto) {

        BusinessException ex = new BusinessException();

        if (entidadePesquisaDto.getUsuarioLogado() == null || entidadePesquisaDto.getUsuarioLogado().getId() == null) {
            ex.addErrorMessage("Usuario logado obrigatório");
            throw ex;
        }

        return entidadeDAO.buscar(entidadePesquisaDto);
    }

    public Long contarPaginado(EntidadePesquisaDto entidadePesquisaDto) {

        BusinessException ex = new BusinessException();

        if (entidadePesquisaDto.getUsuarioLogado() == null || entidadePesquisaDto.getUsuarioLogado().getId() == null) {
            ex.addErrorMessage("Usuario logado obrigatório");
            throw ex;
        }

        return entidadeDAO.contarPaginado(entidadePesquisaDto);
    }

    public Entidade buscarPeloId(Long id) {

        if (id == null) {
            throw new IllegalArgumentException("Parâmetro id não pode ser null");
        }

        BusinessException ex = new BusinessException();

        /* Verificar se existe Entidade com esse id cadastrado */

        Entidade e = entidadeDAO.buscarPeloId(id);

        if (e == null) {
            /* msg cadastrada no arquivo de propriedades */
            ex.addErrorMessage("MN042", id);
            throw ex;
        }
        return e;
    }

    public Entidade incluirAlterar(Entidade entidade, String usuarioLogado) {

        Entidade entidadeRetorno = null;

        validarObjetoNulo(entidade);

        BusinessException ex = new BusinessException();

        if (StringUtils.isBlank(usuarioLogado)) {
            ex.addErrorMessage("Usuario logado obrigatório");
            throw ex;
        }

        vincularUsuarioPessoa(entidade, usuarioLogado);
        if (entidade.getId() == null) {
            entidadeRetorno = entidadeDAO.incluir(entidade, usuarioLogado);
        } else {
            entidadeRetorno = entidadeDAO.alterar(entidade, usuarioLogado);
        }
        return entidadeRetorno;
    }

    private void validarObjetoNulo(Entidade entidade) {
        if (entidade == null) {
            throw new IllegalArgumentException("Parâmetro programa não pode ser null");
        }
    }

    public void excluir(Long id) {

        if (id == null) {
            throw new IllegalArgumentException("Parâmetro id não pode ser null");
        }

        BusinessException ex = new BusinessException();

        /* Verificar se existe Programa com esse id cadastrado */

        Entidade e = genericPersister.findByUniqueProperty(Entidade.class, "id", id);
        if (e == null) {
            /* msg cadastrada no arquivo de propriedades */
            ex.addErrorMessage("MN042", id);
            throw ex;
        }

        entidadeDAO.excluir(e);

    }

    private void vincularUsuarioPessoa(Entidade entidade, String usuarioLogado) {
        List<PessoaEntidade> listaPessoaFinal = new ArrayList<PessoaEntidade>();
        for (PessoaEntidade objPessoa : entidade.getPessoas()) {
            if (objPessoa.getId() == null && objPessoa.getPessoa().getId() == null && objPessoa.getPessoa().isRepresentante()) {
                Usuario usuario = usuarioService.incluirUsuarioRepresentante(objPessoa.getPessoa(), usuarioLogado);
                objPessoa.getPessoa().setUsuario(usuario);
            }
            listaPessoaFinal.add(objPessoa);
        }
        entidade.setPessoas(listaPessoaFinal);
    }

    public void atualizarStatusPessoas() {
        entidadeDAO.atualizarStatusPessoas();
    }

    private void validarLocalEntregaComMesmoNome(Entidade entidade, BusinessException ex) {
        /*
         * Verificar se já existe local de entrega com o mesmo nome cadastrado,
         * pois nesse caso não será permitida a inclusão (regra de negócio)
         */
        for (LocalEntregaEntidade localEntregaEntidade1 : entidade.getLocaisEntregaEntidade()) {
            for (LocalEntregaEntidade localEntregaEntidade2 : entidade.getLocaisEntregaEntidade()) {
                if (localEntregaEntidade1.getNomeEndereco().toUpperCase().equals(localEntregaEntidade2.getNomeEndereco().toUpperCase()) && localEntregaEntidade1.hashCode() != localEntregaEntidade2.hashCode()) {
                    /* msg cadastrada no arquivo de propriedades */
                    ex.addErrorMessage("MN045", localEntregaEntidade2.getNomeEndereco());
                    throw ex;
                }
            }

        }

    }

    private boolean possuiVinculo(LocalEntregaEntidade localEntregaEntidade) {

        return !inscricaoProgramaService.buscarInscricaoLocalEntregaEntidade(localEntregaEntidade).isEmpty();

    }

}
